/*
  delicate-volman - DeLi(cate) Linux Volume Manager
  Copyright (C) 2013 Jaromir Capik <tavvva@email.cz>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <gtk/gtk.h>

#define FORK 1

#define FSTR_LEN  1000
#define FLOPPY_MAX 10
#define CDROM_MAX 10
#define HDD_MAX 20

typedef char fstr[FSTR_LEN+1];


GtkWidget *window;
GtkStatusIcon *trayIcon;

GtkWidget *menu,
	  *menuItemShow,
	  *menuItemHide,
	  *menuItemExit;


GdkPixbuf *pixbuf_cdrom,
	  *pixbuf_floppy,
	  *pixbuf_hdd;

/* CD-ROM */
fstr cdrom[CDROM_MAX];
fstr cdrom_mountpoint[CDROM_MAX];
fstr cdrom_mounttype[CDROM_MAX];
int  cdrom_count;

fstr old_cdrom[CDROM_MAX];
fstr old_cdrom_mountpoint[CDROM_MAX];
fstr old_cdrom_mounttype[CDROM_MAX];
int  old_cdrom_count;

/* Floppy */
fstr floppy[FLOPPY_MAX];
fstr floppy_mountpoint[CDROM_MAX];
fstr floppy_mounttype[CDROM_MAX];
int  floppy_count;

fstr old_floppy[CDROM_MAX];
fstr old_floppy_mountpoint[CDROM_MAX];
fstr old_floppy_mounttype[CDROM_MAX];
int  old_floppy_count;

/* HARD-DRIVE */
fstr hdd[HDD_MAX];
fstr hdd_mountpoint[HDD_MAX];
fstr hdd_mounttype[HDD_MAX];
int  hdd_count;

fstr old_hdd[HDD_MAX];
fstr old_hdd_mountpoint[HDD_MAX];
fstr old_hdd_mounttype[HDD_MAX];
int  old_hdd_count;

/* ------------------------- */

GtkWidget *vbox;

/* CD-ROM */
GtkWidget *hbox_cdrom[CDROM_MAX];
GtkWidget *but_cdrom_mount[CDROM_MAX];
GtkWidget *but_cdrom_umount[CDROM_MAX];
GtkWidget *ico_cdrom[CDROM_MAX];
GtkWidget *lab_cdrom[CDROM_MAX];
GtkWidget *lab_cdrom_mp[CDROM_MAX];
GtkWidget *lab_cdrom_mt[CDROM_MAX];
GtkWidget *sep_cdrom_mp[CDROM_MAX];
GtkWidget *sep_cdrom_mt[CDROM_MAX];
GtkWidget *sep_cdrom_end[CDROM_MAX];
GtkWidget *sep_cdrom_horiz[CDROM_MAX];

/* FLOPPY */
GtkWidget *hbox_floppy[FLOPPY_MAX];
GtkWidget *but_floppy_mount[FLOPPY_MAX];
GtkWidget *but_floppy_umount[FLOPPY_MAX];
GtkWidget *ico_floppy[FLOPPY_MAX];
GtkWidget *lab_floppy[FLOPPY_MAX];
GtkWidget *lab_floppy_mp[FLOPPY_MAX];
GtkWidget *lab_floppy_mt[FLOPPY_MAX];
GtkWidget *sep_floppy_mp[FLOPPY_MAX];
GtkWidget *sep_floppy_mt[FLOPPY_MAX];
GtkWidget *sep_floppy_end[FLOPPY_MAX];
GtkWidget *sep_floppy_horiz[FLOPPY_MAX];

/* HARD-DRIVE */
GtkWidget *hbox_hdd[HDD_MAX];
GtkWidget *but_hdd_umount[HDD_MAX];
GtkWidget *but_hdd_umount_and_sleep[HDD_MAX];
GtkWidget *ico_hdd[HDD_MAX];
GtkWidget *lab_hdd[HDD_MAX];
GtkWidget *lab_hdd_mp[HDD_MAX];
GtkWidget *lab_hdd_mt[HDD_MAX];
GtkWidget *sep_hdd_mp[HDD_MAX];
GtkWidget *sep_hdd_mt[HDD_MAX];
GtkWidget *sep_hdd_end[HDD_MAX];
GtkWidget *sep_hdd_horiz[HDD_MAX];



static void trayShow(GtkMenuItem *item, gpointer user_data);
static void trayHide(GtkMenuItem *item, gpointer user_data);
static void trayExit(GtkMenuItem *item, gpointer user_data);
static void trayIconActivated(GObject *trayIcon, gpointer data);
static void trayIconPopup(GtkStatusIcon *status_icon, guint button, guint32 activate_time, gpointer popUpMenu);
static void destroy (GtkWidget*, gpointer);
static gboolean delete_event (GtkWidget*, GdkEvent*, gpointer);
static gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event, gpointer user_data);


void parse_mount(fstr str,fstr device,fstr mountpoint,fstr type)
{
  int i;
  unsigned char o;
  i = 0;
  strcpy(device,"");
  while ((str[i]!=' ')&&(str[i]!=0x09)&&(str[i]!=0x0A)&&(str[i]!=0x00)) { /* device */
    strncat(device,&str[i],1);i++;
  }
  while ((str[i]==' ')||(str[i]==0x09)) i++;              /* space */
  strcpy(mountpoint,"");
  while ((str[i]!=' ')&&(str[i]!=0x09)&&(str[i]!=0x0A)&&(str[i]!=0x00)) { /* mountpoint */
    if (str[i]!='\\') {
      strncat(mountpoint,&str[i],1);i++;
    } else { /* octal escapes */
      if (strlen(str)>(i+4)) {
        o =(str[++i]-'0')*8*8;
        o+=(str[++i]-'0')*8;
        o+=(str[++i]-'0');
        strncat(mountpoint,(char *)&o,1);i++;
      }
    }
  }
  while ((str[i]==' ')||(str[i]==0x09)) i++;              /* space */
  strcpy(type,"");
  while ((str[i]!=' ')&&(str[i]!=0x09)&&(str[i]!=0x0A)&&(str[i]!=0x00)) { /* type */
    strncat(type,&str[i],1);i++;
  }
}

void get_mount_data(fstr device,fstr mountpoint,fstr type)
{
  FILE *fp;
  fstr str,device_tmp,mountpoint_tmp,type_tmp;
  strcpy(mountpoint,"***not mounted***");
  strcpy(type,"");
  fp = fopen("/proc/mounts","r");
  if (fp != NULL) {
    while(!feof(fp)) {
      strcpy(str,"");
      fgets(str, FSTR_LEN, fp);
      parse_mount(str,device_tmp,mountpoint_tmp,type_tmp);
      if (strcmp(device,device_tmp)==0) {
        strcpy(mountpoint,mountpoint_tmp);
        strcpy(type,type_tmp);
        break;
      }
    }
    fclose(fp);
  }
}


char detect_hdd()
{
  FILE *fp;
  fstr str,device,mountpoint,type;
  int i;

  old_hdd_count = hdd_count;

  for (i=0;i<hdd_count;i++) {
    strcpy(old_hdd[i],hdd[i]);
    strcpy(old_hdd_mountpoint[i],hdd_mountpoint[i]);
    strcpy(old_hdd_mounttype[i],hdd_mounttype[i]);
  }

  hdd_count = 0;

  fp = fopen("/proc/mounts","r");
  if (fp != NULL) {
    while(!feof(fp)) {
      strcpy(str,"");
      fgets(str, FSTR_LEN, fp);
      parse_mount(str,device,mountpoint,type);
      if ((strncmp(mountpoint,"/media/",7)==0) &&
         ((strncmp(device,"/dev/sd",7)==0)||   /* sd* and hd* only */
          (strncmp(device,"/dev/hd",7)==0)))
      {
        strcpy(hdd[hdd_count],device);
        strcpy(hdd_mountpoint[hdd_count],mountpoint);
        strcpy(hdd_mounttype[hdd_count],type);
        hdd_count++;
      }
    }
    fclose(fp);
  }
  if (old_hdd_count!=hdd_count) return 1;
  for (i=0;i<hdd_count;i++) {
    if (strcmp(old_hdd[i],hdd[i])!=0) return 1;
    if (strcmp(old_hdd_mountpoint[i],hdd_mountpoint[i])!=0) return 1;
    if (strcmp(old_hdd_mounttype[i],hdd_mounttype[i])!=0) return 1;
  }

  return 0;
}


void parse_cdrom(char *str)
{
  while ((str[0]!=':')&&(str[0]!=0x00)) str++;
  str++;

  cdrom_count = 0;
  while (strlen(str)>0) {
    while ((str[0]==' ')||(str[0]==0x09)||(str[0]==0x0A)) str++;
    if ((str[0]!=0x0A)&&(strlen(str)>0)) {
      strcpy(cdrom[cdrom_count],"/dev/");
      while ((str[0]!=' ')&&(str[0]!=0x09)&&(str[0]!=0x0A)&&(strlen(str)>0)) {
        strncat(cdrom[cdrom_count],str,1);
        str++;
      }
      cdrom_count++;
    }
    if (str[0]==0x0A) str++;
  }
}

static void mount(GtkWidget *widget, gpointer data)
{
  pid_t pid = fork();
  if(pid == -1) {  /* fork() failed */
    perror("Fork failed!");
    exit(-1);
  } else if(pid == 0) { /* child */
    execlp("delicate-volman-backend","delicate-volman-backend","mount",(char *)data,NULL);
    exit(-1);
  }
}

static void umount(GtkWidget *widget, gpointer data)
{
  pid_t pid = fork();
  if(pid == -1) {  /* fork() failed */
    perror("Fork failed!");
    exit(-1);
  } else if(pid == 0) { /* child */
    execlp("delicate-volman-backend","delicate-volman-backend","umount",(char *)data,NULL);
    exit(-1);
  }
}


char detect_cdrom()
{
  /* FILE *fp; 
  fstr str; */
  int i;

  DIR *dir;
  struct dirent *entry;


  old_cdrom_count = cdrom_count;

  for (i=0;i<cdrom_count;i++) {
    strcpy(old_cdrom[i],cdrom[i]);
    strcpy(old_cdrom_mountpoint[i],cdrom_mountpoint[i]);
    strcpy(old_cdrom_mounttype[i],cdrom_mounttype[i]);
  }

  cdrom_count = 0;

  if ((dir = opendir("/dev/cdroms")) == NULL) {
    return (cdrom_count != old_cdrom_count);
  }

  while ((entry = readdir(dir)) != NULL) {
    if (strcmp(entry->d_name,".") && strcmp(entry->d_name,"..")) {
      sprintf(cdrom[cdrom_count],"/dev/cdroms/%s", entry->d_name);
      cdrom_count++;
    }
  }

  closedir(dir);

/*
  fp = fopen("/proc/sys/dev/cdrom/info","r");
  if (fp != NULL) {
    while(!feof(fp)) {
      strcpy(str,"");
      fgets(str, FSTR_LEN, fp);
      if (strncmp(str,"drive name:",11)==0) {
        parse_cdrom(str);
        break;
      }
    }
    fclose(fp);
  }
*/

  for (i=0;i<cdrom_count;i++) {
    get_mount_data(cdrom[i],cdrom_mountpoint[i],cdrom_mounttype[i]);
  }
  if (old_cdrom_count!=cdrom_count) return 1;
  for (i=0;i<cdrom_count;i++) {
    if (strcmp(old_cdrom[i],cdrom[i])!=0) return 1;
    if (strcmp(old_cdrom_mountpoint[i],cdrom_mountpoint[i])!=0) return 1;
    if (strcmp(old_cdrom_mounttype[i],cdrom_mounttype[i])!=0) return 1;
  }

  return 0;
}


char detect_floppy()
{
  int i;

  DIR *dir;
  struct dirent *entry;

  old_floppy_count = floppy_count;

  for (i=0;i<floppy_count;i++) {
    strcpy(old_floppy[i],floppy[i]);
    strcpy(old_floppy_mountpoint[i],floppy_mountpoint[i]);
    strcpy(old_floppy_mounttype[i],floppy_mounttype[i]);
  }

  floppy_count = 0;
  if ((dir = opendir("/dev/floppy")) == NULL) {
    return (floppy_count != old_floppy_count);
  }

  while ((entry = readdir(dir)) != NULL) {
    if (!strchr(entry->d_name,'u') && strcmp(entry->d_name,".") && strcmp(entry->d_name,"..")) {
      sprintf(floppy[floppy_count],"/dev/floppy/%s", entry->d_name);
      floppy_count++;
    }
  }

  closedir(dir);

  for (i=0;i<floppy_count;i++) {
    get_mount_data(floppy[i],floppy_mountpoint[i],floppy_mounttype[i]);
  }
  if (old_floppy_count != floppy_count) return 1;
  for (i=0;i<floppy_count;i++) {
    if (strcmp(old_floppy[i],floppy[i])!=0) return 1;
    if (strcmp(old_floppy_mountpoint[i],floppy_mountpoint[i])!=0) return 1;
    if (strcmp(old_floppy_mounttype[i],floppy_mounttype[i])!=0) return 1;
  }

  return 0;
}


char detect_all()
{
  char change_detected;
  change_detected = 0;
  if (detect_cdrom()) change_detected = 1;
  if (detect_floppy()) change_detected = 1;
  if (detect_hdd()) change_detected = 1;

  return change_detected;
}


void destroy_cdrom()
{
  int i;
  for (i=0;i<old_cdrom_count;i++) {
    gtk_widget_destroy(GTK_WIDGET(sep_cdrom_end[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_cdrom_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_cdrom_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_cdrom_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_cdrom_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_cdrom[i]));
    gtk_widget_destroy(GTK_WIDGET(ico_cdrom[i]));
    gtk_widget_destroy(GTK_WIDGET(but_cdrom_umount[i]));
    gtk_widget_destroy(GTK_WIDGET(but_cdrom_mount[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_cdrom_horiz[i]));
    gtk_widget_destroy(GTK_WIDGET(hbox_cdrom[i]));
  }
}


void fill_cdrom()
{
  int i;
  for (i=0;i<cdrom_count;i++) {
    hbox_cdrom[i] = gtk_hbox_new (FALSE, 0);
    sep_cdrom_horiz[i] = gtk_hseparator_new();
    but_cdrom_mount[i]  = gtk_button_new_with_label ("MOUNT");
    but_cdrom_umount[i] = gtk_button_new_with_label ("UMOUNT");
    ico_cdrom[i]       = gtk_image_new_from_pixbuf(pixbuf_cdrom);
    lab_cdrom[i]       = gtk_label_new(cdrom[i]);
    lab_cdrom_mp[i]    = gtk_label_new(cdrom_mountpoint[i]);
    lab_cdrom_mt[i]    = gtk_label_new(cdrom_mounttype[i]);
    sep_cdrom_mp[i]    = gtk_vseparator_new();
    sep_cdrom_mt[i]    = gtk_vseparator_new();
    sep_cdrom_end[i]   = gtk_vseparator_new();
    g_signal_connect (G_OBJECT (but_cdrom_mount[i]), "clicked",G_CALLBACK (mount), (gpointer) cdrom[i]);
    g_signal_connect (G_OBJECT (but_cdrom_umount[i]),"clicked",G_CALLBACK (umount), (gpointer) cdrom[i]);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(hbox_cdrom[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(sep_cdrom_horiz[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(but_cdrom_mount[i]) , FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(but_cdrom_umount[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(ico_cdrom[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(lab_cdrom[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(sep_cdrom_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(lab_cdrom_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(sep_cdrom_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(lab_cdrom_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_cdrom[i]), GTK_WIDGET(sep_cdrom_end[i]), FALSE, FALSE, 2);
    gtk_widget_show(GTK_WIDGET(hbox_cdrom[i]));
    gtk_widget_show(GTK_WIDGET(sep_cdrom_horiz[i]));
    gtk_widget_show(GTK_WIDGET(hbox_cdrom[i]));
    gtk_widget_show(GTK_WIDGET(but_cdrom_mount[i]));
    gtk_widget_show(GTK_WIDGET(but_cdrom_umount[i]));
    gtk_widget_show(GTK_WIDGET(ico_cdrom[i]));
    gtk_widget_show(GTK_WIDGET(lab_cdrom[i]));
    gtk_widget_show(GTK_WIDGET(sep_cdrom_mp[i]));
    gtk_widget_show(GTK_WIDGET(lab_cdrom_mp[i]));
    gtk_widget_show(GTK_WIDGET(sep_cdrom_mt[i]));
    gtk_widget_show(GTK_WIDGET(lab_cdrom_mt[i]));
    gtk_widget_show(GTK_WIDGET(sep_cdrom_end[i]));
  }
}


void destroy_floppy()
{
  int i;
  for (i=0;i<old_floppy_count;i++) {
    gtk_widget_destroy(GTK_WIDGET(sep_floppy_end[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_floppy_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_floppy_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_floppy_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_floppy_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_floppy[i]));
    gtk_widget_destroy(GTK_WIDGET(ico_floppy[i]));
    gtk_widget_destroy(GTK_WIDGET(but_floppy_umount[i]));
    gtk_widget_destroy(GTK_WIDGET(but_floppy_mount[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_floppy_horiz[i]));
    gtk_widget_destroy(GTK_WIDGET(hbox_floppy[i]));
  }
}


void fill_floppy()
{
  int i;
  for (i=0;i<floppy_count;i++) {
    hbox_floppy[i] = gtk_hbox_new (FALSE, 0);
    sep_floppy_horiz[i] = gtk_hseparator_new();
    but_floppy_mount[i]  = gtk_button_new_with_label ("MOUNT");
    but_floppy_umount[i] = gtk_button_new_with_label ("UMOUNT");
    ico_floppy[i]       = gtk_image_new_from_pixbuf(pixbuf_floppy);
    lab_floppy[i]       = gtk_label_new(floppy[i]);
    lab_floppy_mp[i]    = gtk_label_new(floppy_mountpoint[i]);
    lab_floppy_mt[i]    = gtk_label_new(floppy_mounttype[i]);
    sep_floppy_mp[i]    = gtk_vseparator_new();
    sep_floppy_mt[i]    = gtk_vseparator_new();
    sep_floppy_end[i]   = gtk_vseparator_new();
    g_signal_connect (G_OBJECT (but_floppy_mount[i]), "clicked",G_CALLBACK (mount), (gpointer) floppy[i]);
    g_signal_connect (G_OBJECT (but_floppy_umount[i]),"clicked",G_CALLBACK (umount), (gpointer) floppy[i]);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(hbox_floppy[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(sep_floppy_horiz[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(but_floppy_mount[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(but_floppy_umount[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(ico_floppy[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(lab_floppy[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(sep_floppy_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(lab_floppy_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(sep_floppy_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(lab_floppy_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_floppy[i]), GTK_WIDGET(sep_floppy_end[i]), FALSE, FALSE, 2);
    gtk_widget_show(GTK_WIDGET(hbox_floppy[i]));
    gtk_widget_show(GTK_WIDGET(sep_floppy_horiz[i]));
    gtk_widget_show(GTK_WIDGET(hbox_floppy[i]));
    gtk_widget_show(GTK_WIDGET(but_floppy_mount[i]));
    gtk_widget_show(GTK_WIDGET(but_floppy_umount[i]));
    gtk_widget_show(GTK_WIDGET(ico_floppy[i]));
    gtk_widget_show(GTK_WIDGET(lab_floppy[i]));
    gtk_widget_show(GTK_WIDGET(sep_floppy_mp[i]));
    gtk_widget_show(GTK_WIDGET(lab_floppy_mp[i]));
    gtk_widget_show(GTK_WIDGET(sep_floppy_mt[i]));
    gtk_widget_show(GTK_WIDGET(lab_floppy_mt[i]));
    gtk_widget_show(GTK_WIDGET(sep_floppy_end[i]));
  }
}



void destroy_hdd()
{
  int i;
  for (i=0;i<old_hdd_count;i++) {
    gtk_widget_destroy(GTK_WIDGET(sep_hdd_end[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_hdd_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_hdd_mt[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_hdd_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_hdd_mp[i]));
    gtk_widget_destroy(GTK_WIDGET(lab_hdd[i]));
    gtk_widget_destroy(GTK_WIDGET(ico_hdd[i]));
    gtk_widget_destroy(GTK_WIDGET(but_hdd_umount_and_sleep[i]));
    gtk_widget_destroy(GTK_WIDGET(but_hdd_umount[i]));
    gtk_widget_destroy(GTK_WIDGET(sep_hdd_horiz[i]));
    gtk_widget_destroy(GTK_WIDGET(hbox_hdd[i]));
  }
}


void fill_hdd()
{
  int i;
  for (i=0;i<hdd_count;i++) {
    hbox_hdd[i] = gtk_hbox_new (FALSE, 0);
    sep_hdd_horiz[i] = gtk_hseparator_new();
    but_hdd_umount[i]= gtk_button_new_with_label ("MOUNT");
    but_hdd_umount_and_sleep[i]= gtk_button_new_with_label ("UMOUNT");
    ico_hdd[i]       = gtk_image_new_from_pixbuf(pixbuf_hdd);
    lab_hdd[i]       = gtk_label_new(hdd[i]);
    lab_hdd_mp[i]    = gtk_label_new(hdd_mountpoint[i]);
    lab_hdd_mt[i]    = gtk_label_new(hdd_mounttype[i]);
    sep_hdd_mp[i]    = gtk_vseparator_new();
    sep_hdd_mt[i]    = gtk_vseparator_new();
    sep_hdd_end[i]   = gtk_vseparator_new();
    g_signal_connect (G_OBJECT (but_hdd_umount[i]), "clicked",G_CALLBACK (mount), (gpointer) hdd[i]);
    g_signal_connect (G_OBJECT (but_hdd_umount_and_sleep[i]), "clicked",G_CALLBACK (umount), (gpointer) hdd[i]);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(hbox_hdd[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(sep_hdd_horiz[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(but_hdd_umount[i]) , FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(but_hdd_umount_and_sleep[i]) , FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(ico_hdd[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(lab_hdd[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(sep_hdd_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(lab_hdd_mp[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(sep_hdd_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(lab_hdd_mt[i]), FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox_hdd[i]), GTK_WIDGET(sep_hdd_end[i]), FALSE, FALSE, 2);
    gtk_widget_show(GTK_WIDGET(hbox_hdd[i]));
    gtk_widget_show(GTK_WIDGET(sep_hdd_horiz[i]));
    gtk_widget_show(GTK_WIDGET(hbox_hdd[i]));
    gtk_widget_show(GTK_WIDGET(but_hdd_umount[i]));
    gtk_widget_show(GTK_WIDGET(but_hdd_umount_and_sleep[i]));
    gtk_widget_show(GTK_WIDGET(ico_hdd[i]));
    gtk_widget_show(GTK_WIDGET(lab_hdd[i]));
    gtk_widget_show(GTK_WIDGET(sep_hdd_mp[i]));
    gtk_widget_show(GTK_WIDGET(lab_hdd_mp[i]));
    gtk_widget_show(GTK_WIDGET(sep_hdd_mt[i]));
    gtk_widget_show(GTK_WIDGET(lab_hdd_mt[i]));
    gtk_widget_show(GTK_WIDGET(sep_hdd_end[i]));
  }
}



void destroy_all()
{
  destroy_floppy();
  destroy_cdrom();
  destroy_hdd();
}

void fill_all()
{
  fill_floppy();
  fill_cdrom();
  fill_hdd();
}


static gboolean time_handler(GtkWidget *widget)
{
  if (GTK_WINDOW(widget) == NULL) return FALSE;

  if (detect_all()) {
    destroy_all();
    fill_all();
  }

  return TRUE;
}

int main(int argc, char *argv[])
{
    int i;

    gtk_init (&argc, &argv);


    detect_cdrom();
    for (i=0;i<cdrom_count;i++) {
      printf("CDROM : %s\n",cdrom[i]);
    }
    cdrom_count=0; /* mandatory */


    detect_floppy();
    for (i=0;i<floppy_count;i++) {
      printf("FLOPPY : %s\n",floppy[i]);
    }
    floppy_count=0; /* mandatory */


    detect_hdd();
    for (i=0;i<hdd_count;i++) {
      printf("HDD : %s\n",hdd[i]);
    }
    hdd_count=0; /* mandatory */


    if (FORK) {
      pid_t pid;
      pid = fork();
      if(pid == -1) {  /* fork() failed */
        perror("Fork failed!");
        exit(-1);
      } else if(pid == 0) { /* child */
      } else {             /* parent */
        fprintf(stderr, "Fork successful (pid = %d)\n", (int)pid);
        exit(0);
      }
    }
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gtk_window_set_title (GTK_WINDOW(window), "DeLi(cate) Linux Volume Manager");
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_widget_set_size_request (window, 600, 400);

    /* icons */
    pixbuf_cdrom = gdk_pixbuf_new_from_file ("/usr/share/delicate-volman/cdrom.png", NULL);
    pixbuf_floppy = gdk_pixbuf_new_from_file ("/usr/share/delicate-volman/floppy.png", NULL);

    /* set tray icon */
    trayIcon  = gtk_status_icon_new_from_pixbuf (pixbuf_cdrom);

    /* set popup menu for tray icon */
    menu = gtk_menu_new();

    menuItemShow = gtk_menu_item_new_with_label ("Show");
    menuItemHide = gtk_menu_item_new_with_label ("Hide");
    menuItemExit = gtk_menu_item_new_with_label ("Exit");
    g_signal_connect (G_OBJECT (menuItemShow), "activate", G_CALLBACK (trayShow), window);
    g_signal_connect (G_OBJECT (menuItemHide), "activate", G_CALLBACK (trayHide), window);
    g_signal_connect (G_OBJECT (menuItemExit), "activate", G_CALLBACK (trayExit), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItemShow);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItemHide);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItemExit);
    gtk_widget_show_all (menu);
    /* set tooltip */
    gtk_status_icon_set_tooltip (trayIcon, "DeLi(cate) Linux Volume Manager");
    /* connect handlers for mouse events */
    g_signal_connect(GTK_STATUS_ICON (trayIcon), "activate", GTK_SIGNAL_FUNC (trayIconActivated), window);
    g_signal_connect(GTK_STATUS_ICON (trayIcon), "popup-menu", GTK_SIGNAL_FUNC (trayIconPopup), menu);
    gtk_status_icon_set_visible(trayIcon, TRUE); /* set icon initially visible */

    g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (destroy), NULL);
    g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK (delete_event), trayIcon);
    g_signal_connect (G_OBJECT (window), "window-state-event", G_CALLBACK (window_state_event), trayIcon);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    g_timeout_add(1000, (GSourceFunc) time_handler, (gpointer) window);

    gtk_main ();

    return 0;
}

static void trayShow(GtkMenuItem *item, gpointer window) 
{
    gtk_widget_show_all (window);
    gtk_window_deiconify(GTK_WINDOW(window));
}

static void trayHide(GtkMenuItem *item, gpointer window) 
{
    gtk_widget_hide(window);
}

static void trayExit(GtkMenuItem *item, gpointer user_data) 
{
    printf("Exit\n");
    gtk_main_quit();
}

static void trayIconActivated(GObject *trayIcon, gpointer window)
{
    if (gtk_window_is_active(window)==TRUE) {
      gtk_widget_hide(window);
    } else {
      gtk_widget_hide(window);
      gtk_widget_show_all (window);
      gtk_window_deiconify(GTK_WINDOW(window));
    }
}

static void trayIconPopup(GtkStatusIcon *status_icon, guint button, guint32 activate_time, gpointer popUpMenu)
{
    gtk_menu_popup(GTK_MENU(popUpMenu), NULL, NULL, gtk_status_icon_position_menu, status_icon, button, activate_time);
}

static void destroy (GtkWidget *window, gpointer data)
{
    gtk_main_quit ();
}

static gboolean delete_event (GtkWidget *window, GdkEvent *event, gpointer data)
{
    gtk_widget_hide(GTK_WIDGET(window)); /* minimize window */
    return TRUE;                         /* disable window deletion */
}


static gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event, gpointer trayIcon)
{
    return TRUE;
}

